# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from jmb.filters.admin import AdvancedSearchModelAdmin
from mptt.admin import DraggableMPTTAdmin
from thx.core.admin import UserAdminMixin

from thx.role.admin_links import ImportExportRoleLinkMixin

from . import models


class ProfileAdmin(UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin):
    search_fields = ('codename', 'name',)
    advanced_search_fields = (
        ('status',),
        ('name__icontains', 'codename__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                ('codename', 'name'),
                ('users', 'roles',)
            ),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('pk', 'codename', 'name', 'status', 'ordering')
    list_display_links = ('codename', 'name',)
    filter_horizontal = ('users', 'roles',)
    readonly_fields = ('codename', 'creator', 'date_create', 'last_modifier', 'date_last_modify')


class RoleAdmin(ImportExportRoleLinkMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin):
    search_fields = ('profiles__name',)
    advanced_search_fields = (
        ('status',),
        ('profiles__name__icontains',),
        ('app_label__icontains', 'model_name__icontains', 'action__icontains',),
        ('profiles__users__email__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                ('app_label', 'model_name', 'action',),
                # ('profiles',),
            )
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'app_label', 'model_name', 'action', 'ordering')
    list_display_links = ('id', 'status', 'app_label', 'model_name',)
    # filter_horizontal = ('profiles',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class MenuTranslationInline(admin.TabularInline):
    model = models.MenuTranslation
    fields = ('title', 'description', 'language_code')
    extra = 0


class MenuAdmin(
    DraggableMPTTAdmin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('title', 'description')
    advanced_search_fields = (
        ('title__icontains', 'description__icontains',),
        ('parent__title__icontains', 'parent__description__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                'parent',
                ('title', 'description',),
                'router',
                ('profiles', 'role_list',),
                'info',
            ),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('status',),
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = ('tree_actions', 'menu_title', 'id', 'status', 'title', 'profiles_list',)
    list_display_links = ('menu_title',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify', 'role_list',)
    autocomplete_fields = ('parent',)
    filter_horizontal = ('profiles',)
    inlines = (MenuTranslationInline, )

    def menu_title(self, instance):
        return format_html(
            '<div style="text-indent:{}px">{}</div>',
            instance._mpttfield('level') * self.mptt_level_indent,
            instance.title,  # Or whatever you want to put here
        )


class MenuTranslationAdmin(UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin):
    """
    To allow the overriding of change_list_template, add 'mptt' in INSTALLED_APPS before 'django.contrib.admin'.
    """
    mptt_indent_field = "parent"
    search_fields = ('title', 'description')
    advanced_search_fields = (
        ('title__icontains', 'description__icontains',),
        ('menu__title__icontains', 'menu__description__icontains', ),
    )
    list_filter = ('status', 'language_code',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                'menu', 'title', 'description', 'language_code'
            ),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status',),
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = ('id', 'language_code', 'title', 'description', 'status', 'ordering',)
    list_display_links = ('id', 'language_code', 'title',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)
    autocomplete_fields = ('menu',)


admin.site.register(models.Profile, ProfileAdmin)
admin.site.register(models.Role, RoleAdmin)
admin.site.register(models.Menu, MenuAdmin)
admin.site.register(models.MenuTranslation, MenuTranslationAdmin)
