# -*- coding: utf-8 -*-
from django.conf import settings
from rest_framework.permissions import IsAuthenticated

from thx.role import models


class ThxRolePermission(IsAuthenticated):
    """Permission class for role-based permissions in the thx-role app.

    This permission class extends the `IsAuthenticated` permission provided by Django Rest Framework and adds custom
    logic for role-based permissions. It checks if the user has the required role permissions to perform the requested
    action. The permission is granted if the user is authenticated and either has the superuser role or the required
    role for the specified action.

    By default, the superuser is always allowed access, but this behavior can be modified by setting
    `always_allow_superuser` to False. Similarly, staff members can be required to have the staff profile by setting
    `check_staff_profile` to True.

    Subclasses of `ThxRolePermission` can override the `default_has_permission` method to provide custom default
    permissions checking.

    Attributes:
        always_allow_superuser (bool): Set as False if the superuser needs to be assigned to the superuser profile in
        the Profile table.
        check_staff_profile (bool): Set as True if staff needs to be assigned to the staff profile in the Profile table.

    """

    always_allow_superuser = True
    check_staff_profile = settings.THX_ROLE_STAFF_PROFILE_NEEDS_CHECK

    def has_permission(self, request, view):
        """
        Return `True` if permission is granted, `False` otherwise.

        Overrides the `has_permission` method of the `IsAuthenticated` class to add role-based permission checking.

        :param request: the restframework request
        :param view: the restframework view
        :return: boolean indicating if the user has permission
        """
        has_permission = super().has_permission(request, view)
        if has_permission:
            has_permission = (
                (self.always_allow_superuser and request.user.is_superuser)
                or self.has_role_permission(request, view)
            )
            if not has_permission:
                return False

            has_permission, permission_checked = view.ar.has_permission(request, view)
            if not permission_checked:
                return self.default_has_permission(request, view)
        return has_permission

    def has_role_permission(self, request, view):
        """Check if the user has the required role permission.

        This method checks if the user has the required role permission to perform the requested action.
        If the `THX_ROLE_CHECK_PERMISSIONS` setting is True and the `check_role_permissions` attribute of the
        `ApiRole` instance is True, it queries the database for matching roles and profiles.

        :param request: the restframework request
        :param view: the restframework view
        :return: boolean indicating if the user has the required role permission
        """
        if self.check_default_profiles(request, view):
            return True

        if settings.THX_ROLE_CHECK_PERMISSIONS and view.ar.check_role_permissions:
            objs = models.Role.objects.filter(
                status=1,
                app_label=view.ar.app_label,
                model_name=view.ar.model_name,
                action=view.action,
                profiles__users=request.user,
            )
            if view.ar.profile:
                objs = objs.filter(profiles__codename=view.ar.profile)
            return objs.only("pk").exists()
        return True

    def check_default_profiles(self, request, view):
        """Check default profiles to grant permission.

        This method checks if the user has the superuser or staff profile, and if so, grants permission.

        :param request: the restframework request
        :param view: the restframework view
        :return: boolean indicating if the user has the superuser or staff profile
        """
        if view.ar.profile == "superuser" and request.user.is_superuser:
            return True
        if not self.check_staff_profile and view.ar.profile == "staff" and request.user.is_staff:
            return True
        return False

    def default_has_permission(self, request, view):
        """Default method for checking default permissions.

        Subclasses should implement this method to provide custom default permissions checking.

        :param request: the restframework request
        :param view: the restframework view
        :return: boolean indicating if the user has default permission
        """
        return True
