# -*- coding: utf-8 -*-
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from thx.api.serializers import ResponseSerializerMixin


class ThuxRoleSerializerMixin(ResponseSerializerMixin):
    """Mixin for custom serializers in the thx-role Django application.

    This mixin extends the `ResponseSerializerMixin` to provide custom behavior for role-based serializers.
    It includes methods for determining field names based on role permissions and validating data based on
    required fields and status.

    Attributes:
        view (rest_framework.views.APIView): The view associated with the serializer.

    """

    @property
    def view(self):
        """Get the associated view from the serializer context.

        :return: rest_framework.views.APIView: The view associated with the serializer.
        :raises ImproperlyConfigured: If the view is not found in the serializer context.
        """
        try:
            return self.context['view']
        except KeyError:
            raise ImproperlyConfigured(
                'Using ThuxRoleSerializerMixin, the view is always required in the serializer context.'
            )

    def get_field_names(self, *args, **kwargs):
        """Get the field names based on role permissions.

        By default, if the user is a superuser, all field names are returned. Otherwise, the field names are filtered
        based on the `used_fields` attribute of the associated `ApiRole` instance.

        :param args: Variable length argument list.
        :param kwargs: Arbitrary keyword arguments.
        :return: List[str]: The filtered list of field names.
        """
        # TODO: fixMe. dobbiamo permettere di definire il default diverso da __all__ e permettere di sovrascrivere
        #  la gestione tramite adapter
        view = self.view
        fields = view.ar.used_fields
        if fields is None:
            return super().get_field_names(*args, **kwargs)
        if fields == serializers.ALL_FIELDS:
            return super().get_field_names(*args, **kwargs)
        return fields

    def validate(self, data):
        """Validate the data based on required fields and status.

        This method checks if all the required fields specified by the `required_fields` attribute of the associated
        `ApiRole` instance are present in the data. It also validates the `status` field based on the `from_status` and
        `to_status` attributes of the `ApiRole` instance.

        :param data: The data to validate.
        :return: dict: The validated data.
        :raises serializers.ValidationError: If any required field is missing or the status is invalid.
        """
        view = self.view
        for field in view.ar.required_fields:
            if data.get(field, None) is None:
                raise serializers.ValidationError({field: _("field is required")})
        validate = super().validate(data)

        status = data.get('status', None)
        if status and (view.ar.from_status is not None or view.ar.to_status):
            if status != view.ar.from_status and status not in view.ar.to_status:
                raise serializers.ValidationError({'status': _("invalid status")})
        return validate

    def to_representation(self, instance):
        """Convert the instance to its serialized representation.

        This method uses the `get_response_serializer_class` method of the associated `ApiRole` instance to determine
        the response serializer for the instance. If a response serializer is found, it is used to serialize the
        instance. Otherwise, the default behavior of `ResponseSerializerMixin` is used.

        :param instance: The instance to serialize.
        :return: dict: The serialized representation of the instance.
        """
        response_serializer = self.view.ar.get_response_serializer_class(self)
        if response_serializer:
            return response_serializer(instance, context=self.context).data
        return super().to_representation(instance)
