# -*- coding: utf-8 -*-

import django
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models

from thx.role import managers

if django.VERSION < (3, 1):
    from django.contrib.postgres.fields import JSONField
else:
    from django.db.models import JSONField

from django.utils.translation import gettext_lazy as _
from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel

User = get_user_model()


class Profile(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    name = models.CharField(
        unique=True,
        max_length=200,
        verbose_name=_('name'),
    )
    codename = models.CharField(
        max_length=255,
        unique=True,
        db_index=True,
        verbose_name=_('codename'),
    )
    users = models.ManyToManyField(
        User,
        related_name="profiles",
        blank=True,
        verbose_name=_('users'),
    )
    roles = models.ManyToManyField(
        "Role",
        blank=True,
        related_name="profiles",
        verbose_name=_("roles"),
    )

    class Meta:
        verbose_name = _('profile')
        verbose_name_plural = _('profiles')
        ordering = ('ordering',)
        permissions = (
            ("list_profile", "Can list profile"),
            ("detail_profile", "Can detail profile"),
            ("enable_profile", "Can enable profile"),
            ("disable_profile", "Can disable profile"),
        )

    def __str__(self):
        return self.name

    @classmethod
    def add_profile_user(cls, codename, user):
        cls.objects.get(codename=codename).add_user(user)

    def add_user(self, user):
        self.users.add(user)


class Role(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    @classmethod
    def get_roles(cls, user):
        return Role.objects.filter(profiles__users=user).distinct()

    app_label = models.CharField(
        max_length=100,
        verbose_name=_('app label')
    )
    model_name = models.CharField(
        max_length=100,
        verbose_name=_('model name')
    )
    action = models.CharField(
        max_length=50,
        verbose_name=_('action'),
        help_text=_('e.g.: "create", "enable", "destroy"'),
    )

    class Meta:
        verbose_name = _('role')
        verbose_name_plural = _('roles')
        # TODO: refactor using UniqueConstraint
        unique_together = (('app_label', 'model_name', 'action'),)
        ordering = ('ordering', 'app_label', 'model_name', 'action')
        permissions = (
            ("list_role", "Can list role"),
            ("detail_role", "Can detail role"),
            ("enable_role", "Can enable role"),
            ("disable_role", "Can disable role"),
        )

    def __str__(self):
        return "{} | {} | {}".format(
            self.app_label,
            self.model_name,
            self.action,
        )

    def clean(self):
        self.set_app_label()
        self.set_model_name()
        self.set_action()
        super().clean()

    def set_app_label(self):
        self.app_label = self.app_label.lower()

    def set_model_name(self):
        self.model_name = self.model_name.lower()

    def set_action(self):
        self.action = self.action.lower()

    @property
    def role_permission(self) -> str:
        return f"{self.app_label}-{self.model_name}-{self.action}"


class Menu(MPTTModel, CleanModel, UserModel, DateModel, StatusModel):
    parent = TreeForeignKey(
        "self",
        on_delete=models.CASCADE,
        related_name='children',
        null=True, blank=True,
        verbose_name=_('parent'),
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('title'),
    )
    description = models.CharField(
        max_length=255,
        null=True, blank=True,
        verbose_name=_('description'),
    )
    router = models.CharField(
        max_length=255,
        verbose_name=_('router'),
    )
    profiles = models.ManyToManyField(
        Profile,
        related_name='menus',
        blank=True,
        verbose_name=_('profiles'),
    )
    info = JSONField(
        blank=True, null=True,
        verbose_name=_('info'),
    )

    objects = managers.MenuManager()

    class Meta:
        verbose_name = _('menu')
        verbose_name_plural = _('menu')
        permissions = (
            ("list_menu", "Can list menu"),
            ("detail_menu", "Can detail menu"),
            ("enable_menu", "Can enable menu"),
            ("disable_menu", "Can disable menu"),
        )

    class MPTTMeta:
        parent_attr = 'parent'

    def __str__(self):
        return f"{self.parent} | {self.title}" if self.parent else f"{self.title}"

    def role_list(self):
        return [obj.role_permission for obj in Role.objects.filter(profiles__menus=self, status=1)]

    def profiles_list(self):
        return [obj.codename for obj in self.profiles.filter(status=1)]

    @property
    def parent_title(self):
        return self.parent.title if self.parent else None


class MenuTranslation(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    menu = models.ForeignKey(
        Menu,
        on_delete=models.CASCADE,
        related_name="translations",
        verbose_name=_("menu")
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('title'),
    )
    description = models.CharField(
        max_length=255,
        null=True, blank=True,
        verbose_name=_('description'),
    )
    language_code = models.CharField(
        max_length=2,
        choices=settings.LANGUAGES,
        verbose_name=_('language code'),
    )

    class Meta:
        verbose_name = _('menu translation')
        verbose_name_plural = _('menu translation')
        ordering = ('ordering', 'menu', 'language_code',)
        permissions = (
            ("list_menutranslation", "Can list menu translation"),
            ("detail_menutranslation", "Can detail menu translation"),
            ("enable_menutranslation", "Can enable menu translation"),
            ("disable_menutranslation", "Can disable menu translation"),
        )
