# -*- coding: utf-8 -*-
import logging
from typing import Any, List, Tuple

from django.apps import apps
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db.models import QuerySet
from django.urls import URLPattern, URLResolver
from django.utils.module_loading import import_string

from . import models
from .core.adapters import BaseRoleAdapter

User = get_user_model()

logger = logging.getLogger('console')


class ApiRole:
    """ApiRole class to manage role-based permissions for API views."""

    profile: str
    app_label: str
    model_name: str

    def __init__(self, request, view, *args, **kwargs):
        """ApiRole constructor.

        :param request: the restframework request
        :param view: the restframework view
        :param args: the view positional arguments
        :param kwargs: view kwargs
        """
        self.request = request
        self.view = view
        self.args = args
        self.kwargs = kwargs

        # Initialize API ROLE components
        self.profile = self.kwargs.get('profile', '')
        self.app_label = self.view.queryset.model._meta.app_label  # noqa
        self.model_name = self.view.queryset.model._meta.model_name  # noqa
        self.adapter = self.get_adapter()

        self.instance = self.get_object()
        self.from_status = self.get_from_status()
        self.to_status = self.get_to_status()
        self.required_fields = self.get_required_fields()
        self.used_fields = self.get_used_fields()

    def get_adapter(self, app_label=None, model_name=None) -> BaseRoleAdapter:
        """Load the adapter class.

        Try to load the settings: settings.THX_ROLE_{app_label.upper()}_{model_name.upper()}_ADAPTER.
        If not defined, use the default adapter: settings.THX_ROLE_DEFAULT_ADAPTER

        :param app_label: the app label (default None)
        :param model_name: the model name (default None)
        :return: BaseRoleAdapter instance
        """
        app_label = app_label or self.app_label
        model_name = model_name or self.model_name
        return import_string(
            getattr(
                settings,
                f"THX_ROLE_{app_label.upper()}_{model_name.upper()}_ADAPTER",
                settings.THX_ROLE_DEFAULT_ADAPTER
            )
        )(self)

    def has_permission(self, request, view) -> Tuple[bool, bool]:
        """Return True or False if the user has permission to perform the action.

        :param request: the restframework request
        :param view: the restframework view
        :return: Tuple[bool, bool]: has_permission, permission_checked.
        """
        return self.adapter.has_permission(request, view)

    def filter_queryset(self, request, queryset, view) -> Tuple[QuerySet, bool]:
        """Filter the queryset based on the user's role permissions.

        :param request: the restframework request
        :param queryset: the queryset to filter
        :param view: the restframework view
        :return: Tuple[bool, bool]: QuerySet, queryset_filtered.
        """
        return self.adapter.filter_queryset(request, queryset, view)

    def get_serializer_class(self):
        """Get the serializer class based on the user's role.

        :return: the serializer class
        """
        return self.adapter.get_serializer_class()

    def get_response_serializer_class(self, serializer):
        """Get the response serializer class based on the user's role and action.

        :param serializer: the current serializer
        :return: the response serializer class
        """
        return self.adapter.get_response_serializer_class(serializer)

    def get_object(self):
        """Get the object based on the URL lookup parameter.

        :return: the retrieved object or None
        """
        qs = self.view.get_queryset()
        try:
            lookup_url_kwarg = self.view.lookup_url_kwarg or self.view.lookup_field
            filter_kwargs = {self.view.lookup_field: self.kwargs[lookup_url_kwarg]}
            return qs.get(**filter_kwargs)
        except Exception:
            # TODO: handle the correct exceptions
            pass

    def get_required_fields(self) -> List[str]:
        """Return user required fields for app, model, and from_status assigned.

        :return: list of required fields
        """
        return self.adapter.get_required_fields()

    def get_used_fields(self):
        """Return user used fields for app, model, from status, and action assigned.

        :return: list of used fields
        """
        return self.adapter.get_used_fields()

    def get_from_status(self) -> Any:
        """Return list of user "from status" for app, model, and from status assigned.

        :return: list of "from status"
        """
        return self.adapter.get_from_status(self.instance)

    def get_to_status(self) -> List[Any]:
        """Return list of user "to status" for app, model, and from status assigned.

        :return: list of "to status"
        """
        return self.adapter.get_to_status(self.from_status)

    @property
    def check_role_permissions(self) -> bool:
        """Property that checks indicates if profile-role based permissions checking must be done.

        :return: bool, profile-role checking must be done
        """
        return self.adapter.check_role_permissions

    def get_all_roles(self):
        """Get all roles associated with the user.

        :return: queryset of roles
        """
        return models.Role.objects.filter(
            profiles__users=self.user,
            status=1,
        ).distinct()

    def get_all_profiles(self):
        """Get all profiles associated with the user.

        :return: list of profile codenames
        """
        return models.Profile.objects.filter(
            users=self.user,
            status=1,
        ).values_list('codename', flat=True).order_by('codename').distinct('codename')

    def get_app_profiles(self):
        """Get profiles associated with the user for a specific app.

        :return: list of profile codenames
        """
        return models.Profile.objects.filter(
            users=self.user,
            roles__app_label=self.app_label,
            status=1,
        ).values('codename').order_by('codename').distinct('codename')

    def get_roles(self):
        """Get user roles for a specific app and model.

        :return: queryset of roles
        """
        return models.Role.objects.filter(
            app_label=self.app_label,
            model_name=self.model_name,
            profiles__users=self.user,
            status=1,
        ).distinct()

    def get_role_codenames(self):
        """Get user role codenames for a specific app and model.

        :return: list of role codenames
        """
        # TODO: deprecation warning
        return self.get_profile_codenames()

    def get_profile_codenames(self):
        """Get user profile codenames for a specific app and model.

        :return: list of profile codenames
        """
        return list(self.get_roles().values_list('profiles__codename', flat=True).distinct())

    def write_log(self):
        """Write debug information to the logger."""
        logger.debug('-----------------------------------')
        logger.debug(' API ROLE ')
        logger.debug('-----------------------------------')
        logger.debug('app label:        %s', self.app_label)
        logger.debug('model name:       %s', self.model_name)
        logger.debug('model class:      %s', self.view.queryset.model.__name__)
        logger.debug('action:           %s', self.view.action)
        logger.debug('has permission:   %s', self.has_permission(self.request, self.view))
        logger.debug('role codename:    %s', self.profile)
        logger.debug('-----------------------------------')
        logger.debug(' SERIALIZER ')
        logger.debug('-----------------------------------')
        logger.debug('instance:         %s', self.instance)
        logger.debug('from status:      %s', self.from_status)
        logger.debug('to status:        %s', self.to_status)
        logger.debug('used fields:      %s', self.used_fields)
        logger.debug('required fields:  %s', self.required_fields)
        logger.debug('-----------------------------------')


class InitialRole:
    """InitialRole class to manage initial role and profile creation for users."""

    def __init__(self, user_id=1, profiles=settings.THX_ROLE_PROFILES, app_label=None):
        """InitialRole constructor.

        :param user_id: the user ID (default 1)
        :param profiles: list of profiles (default settings.THX_ROLE_PROFILES)
        :param app_label: the app label (default None)
        """
        self.user = User.objects.get(pk=user_id)
        self.profiles = profiles
        root_urlconf = __import__(settings.ROOT_URLCONF, {}, {}, [''])
        self.roles = self.find_roles(root_urlconf.urlpatterns, app_label)

    def find_roles(self, urlpatterns, app_label=None):
        """Find roles associated with views in urlpatterns.

        :param urlpatterns: the list of URL patterns.
        :param app_label: the app label to limit urlpatterns (default None)
        :return: list of roles to be generated.
        """
        from thx.role.core.views import ThuxRoleViewMixin, ThuxRoleViewSetMixin

        default_actions = ['list', 'retrieve', 'create', 'update', 'partial_update', 'destroy']
        inspected_views = []
        roles = []
        for pattern in urlpatterns:
            if isinstance(pattern, URLPattern):
                if hasattr(pattern.callback, 'cls'):
                    if (
                        issubclass(pattern.callback.cls, ThuxRoleViewSetMixin)
                        or issubclass(pattern.callback.cls, ThuxRoleViewMixin)
                    ):
                        if pattern.callback.cls not in inspected_views:
                            view_instance = pattern.callback.cls()
                            inspected_views.append(pattern.callback.cls)

                            # Check if the callback is based on ThuxRoleViewSetMixin class
                            _app_label = view_instance.queryset.model._meta.app_label  # noqa
                            if app_label and app_label != _app_label:
                                continue

                            model_name = view_instance.queryset.model._meta.model_name  # noqa

                            # Default actions
                            for action in default_actions:
                                try:
                                    getattr(view_instance, action)
                                    roles.append((_app_label, model_name, action))
                                except AttributeError:
                                    continue

                            # Extra actions
                            if hasattr(view_instance, 'get_extra_actions'):
                                for drf_action in view_instance.get_extra_actions():
                                    for action in set(drf_action.mapping.values()):
                                        roles.append((_app_label, model_name, action))

            elif isinstance(pattern, URLResolver):
                roles.extend(self.find_roles(pattern.url_patterns, app_label))
        return roles

    def insert_all(self):
        """Insert all profiles and roles.
        This method must be used inside migrations, keep it safe.

        :return: tuples of inserted profiles and roles
        """
        profiles = self.insert_profiles()
        roles = self.insert_roles()
        return profiles, roles

    def insert_profiles(self):
        """Insert all profiles.
        This method must be used inside migrations, keep it safe.

        :return: list of inserted profiles
        """
        Profile = apps.get_model("role", "Profile")  # noqa
        return [
            Profile.objects.get_or_create(
                name=profile, codename=profile_codename,
                defaults=dict(
                    name=profile, codename=profile_codename,
                    creator=self.user, last_modifier=self.user,
                )
            ) for profile, profile_codename in self.profiles
        ]

    def insert_roles(self):
        """Insert roles for the user.
        This method must be used inside migrations, keep it safe.

        :return: list of inserted roles
        """
        Role = apps.get_model("role", "Role")  # noqa
        return [
            Role.objects.get_or_create(
                app_label=app_label, model_name=model_name, action=action,
                defaults=dict(
                    app_label=app_label, model_name=model_name, action=action,
                    creator=self.user, last_modifier=self.user,
                )
            ) for app_label, model_name, action in self.roles
        ]
